/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package components

import (
	settings "k8s.libre.sh/application/settings"
	meta "k8s.libre.sh/meta"
	"k8s.libre.sh/objects"
	"k8s.libre.sh/objects/container"
	deployment "k8s.libre.sh/objects/deployment"
	"k8s.libre.sh/objects/pod"
	service "k8s.libre.sh/objects/service"

	corev1 "k8s.io/api/core/v1"
)

const DefaultComponent string = "app"

// +kubebuilder:object:generate=true
type Workload struct {
	*meta.ObjectMeta       `json:"commonMeta,omitempty"`
	*deployment.Deployment `json:"deployment,omitempty"`
	*Network               `json:"network,omitempty"`
	Settings               []string `json:"settings,omitempty"`
}

func (w *Workload) TemplateValues() map[string]interface{} {
	values := make(map[string]interface{})
	values["meta"] = w.ObjectMeta.TemplateValues()
	values["service"] = w.Network.TemplateValues()
	values["deployment"] = w.Deployment.TemplateValues()
	return values
}

func (w *Workload) HasSettings() []string {
	return w.Settings
}

func (w *Workload) GetMeta() meta.Instance {
	return w.ObjectMeta
}

func (w *Workload) GetObjects() map[int]objects.Object {

	objects := make(map[int]objects.Object)

	/* 	ing := &Ingress{
	   		// TODO TO FIX
	   		ObjectMeta: w.IngressMeta,
	   		Backends: []Backend{
	   			*w.Backend,
	   		},
	   	}

	   	srv := &Service{
	   		// TODO TO FIX
	   		ObjectMeta: w.ServiceMeta,
	   		Backend:    w.Backend,
	   	}
	*/

	srv := w.Network.GetService()
	ing := w.Network.GetIngress()

	// TODO TO FIX
	meta.SetObjectMeta(w.ObjectMeta, ing.ObjectMeta)
	meta.SetObjectMeta(w.ObjectMeta, srv.ObjectMeta)

	objects[0] = srv
	objects[1] = ing
	objects[2] = w.Deployment

	return objects
}

func (w *Workload) SetDependencies(s *settings.Component) { w.ConfigSpec = s.ConfigSpec }
func (w *Workload) GetDependencies() settings.Config      { return w.ConfigSpec }

func (w *Workload) SetDefaults() {

	// TODO FIXME
	if len(w.ServiceMeta.GetName()) > 0 {
		w.HTTPBackend.ServiceName = w.ServiceMeta.GetName()
	} else {
		w.HTTPBackend.ServiceName = w.ObjectMeta.GetName()
	}

	containerPort := corev1.ContainerPort{
		Name:          w.HTTPBackend.Port.Name,
		ContainerPort: w.HTTPBackend.Port.Port,
		Protocol:      w.HTTPBackend.Port.Protocol,
	}

	w.ContainerSpec.Ports = append(w.ContainerSpec.Ports, containerPort)

}

func (w *Workload) Init() {

	// TODO improve initialisation of objectMeta, labels...
	if w.ObjectMeta == nil {
		w.ObjectMeta = &meta.ObjectMeta{}
	}

	if len(w.ObjectMeta.Labels) == 0 {
		w.ObjectMeta.Labels = make(map[string]string)
	}

	// TODO FIXME
	if len(w.ObjectMeta.GetComponent()) == 0 {
		w.ObjectMeta.SetComponent(DefaultComponent)
	}

	if w.Deployment == nil {
		w.Deployment = &deployment.Deployment{}
	}
	if w.DeploymentSpec == nil {
		w.DeploymentSpec = &deployment.DeploymentSpec{}
	}
	if w.PodSpec == nil {
		w.PodSpec = &pod.PodSpec{}
	}
	if w.ContainerSpec == nil {
		w.ContainerSpec = &container.ContainerSpec{}
	}
	if w.Network == nil {
		w.Network = &Network{}
	}

	if w.Deployment.ObjectMeta == nil {
		w.Deployment.ObjectMeta = &meta.ObjectMeta{}
	}

	if w.ConfigSpec == nil {
		w.ConfigSpec = &settings.ConfigSpec{}
	}

	if w.HTTPBackend == nil {
		w.HTTPBackend = &HTTPBackend{}
	}

	if w.HTTPBackend.ServiceSpec == nil {
		w.HTTPBackend.ServiceSpec = &service.ServiceSpec{}
	}

	if w.ServiceMeta == nil {
		w.ServiceMeta = &meta.ObjectMeta{}
	}

	if w.IngressMeta == nil {
		w.IngressMeta = &meta.ObjectMeta{}
	}
}
