/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package application

import (
	settings "k8s.libre.sh/application/settings"
	"k8s.libre.sh/application/settings/parameters"
	"k8s.libre.sh/interfaces"
	"k8s.libre.sh/meta"
)

type Instance interface {
	meta.Instance
	GetComponents() map[string]Component
	//	SyncOrder() map[int]string
	GetOwner() interfaces.Object
	GetSettings() map[string]settings.Settings
	SetDefaults()
}

type InstanceReconciler struct {
	ReconcilerBase *ReconcilerBase
	Instance       Instance
}

// func GetTemplateValues(i Instance) map[string]interface{} {
func GetTemplateValues(i Instance, setts map[string]settings.Settings) map[string]interface{} {
	values := make(map[string]interface{})
	cptsVals := make(map[string]interface{})
	settsVals := make(map[string]interface{})

	for _, cpt := range i.GetComponents() {
		cptsVals[cpt.GetComponent()] = cpt.TemplateValues()
	}

	for _, sett := range setts {
		// TODO create templateValues for Settings interface
		settsVals[sett.GetMeta().GetComponent()] = parameters.KeyPairValues(sett.GetConfig().GetParameters())
	}

	values["components"] = cptsVals
	values["settings"] = settsVals

	return values

}

func CreateAndInitSettings(i Instance, r interfaces.Reconcile) (setts map[string]settings.Settings, err error) {

	setts = i.GetSettings()

	// First we init external values and  random values
	for _, sett := range setts {
		err = sett.Init(r, i.GetOwner())

		if err != nil {
			return setts, err
		}

	}

	// Transform instance to template values used to init settings
	values := GetTemplateValues(i, setts)

	// Init the values from templates
	for _, sett := range setts {
		sett.GetConfig().InitTemplateValues(values)
	}

	return setts, err
}

// TODO Transform to createandinit and return application object so we can store the values
func Init(i Instance, r interfaces.Reconcile) error {
	return nil
}
