/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package meta

import (
	"k8s.io/apimachinery/pkg/labels"
	interfaces "k8s.libre.sh/interfaces"
)

// Meta interface is a stripped down version of https://godoc.org/k8s.io/apimachinery/pkg/apis/meta/v1#Object with
// only user definied specs.
//
// To mutates an object a type must implements this interface
type Meta interface {
	GetLabels() map[string]string
	SetLabels(labels map[string]string)
	GetAnnotations() map[string]string
	SetAnnotations(annotations map[string]string)
	GetName() string
	SetName(string)
	GetNamespace() string
	SetNamespace(string)
}

// MutateMeta mutates an Object meta
func MutateMeta(o Meta, obj interfaces.Object) error {
	obj.SetLabels(o.GetLabels())

	if len(o.GetAnnotations()) > 0 {
		obj.SetAnnotations(labels.Merge(obj.GetAnnotations(), o.GetAnnotations()))
	}

	return nil
}
