/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package deployment

import (
	pod "k8s.libre.sh/objects/pod"

	appsv1 "k8s.io/api/apps/v1"
)

type Mutate interface {
	GetReplicas() *int32
	GetDeploymentStrategy() appsv1.DeploymentStrategy
	pod.Mutate
}

func MutateDeployment(i Mutate, obj *appsv1.Deployment) error {

	obj.Spec.Replicas = i.GetReplicas()
	obj.Spec.Strategy = i.GetDeploymentStrategy()

	err := pod.MutatePod(i, &obj.Spec.Template)
	if err != nil {
		return err
	}

	return nil
}
